declare namespace tei="http://www.tei-c.org/ns/1.0";
declare function local:term-callback($term, $data) {
 <term freq="{$data[1]}" docs="{$data[2]}">{$term}</term>
};

let $callback := util:function(xs:QName('local:term-callback'), 2)
let $docs := doc('/db/test/test.xml')
let $rdgs := for $rdg in $docs//tei:listWit//tei:witness/@xml:id return concat('#', $rdg)
let $pool := $docs//tei:p
let $query := 'text'

let $commonTerms := 
  let $nodes := 
    for $a in $pool
    (: here nodes can be refined first by querying, if required :)
    return $a(:[ft:query(., $query)]:)
  return
    for $term in util:index-keys($nodes, '', $callback, 15000, 'lucene-index')
    return <term>{
      (
      for $att in $term/@*
      return attribute {$att/name()} {$att * count($rdgs)}
      ,
      $term/node()
      )
    }</term>

let $rdgTerms := 
  for $rdg in $rdgs
  let $nodes := 
    for $a in $pool//tei:rdg[tokenize(string(@wit), '\s+') = $rdg]
    (: here nodes can be refined first by querying, if required :)
    return $a(:[ft:query(., $query)]:)
  return
    for $term in util:index-keys($nodes, '', $callback, 15000, 'lucene-index')
    return <term wit="{$rdg}">{
      $term/(@*, node())
    }</term>

let $conflateXSLT := 
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
    xmlns:xs="http://www.w3.org/2001/XMLSchema"
    exclude-result-prefixes="xs"
    version="2.0">    
    <xsl:template match="terms">
        <xsl:for-each-group select="term" group-by=".">
            <term>
                <xsl:for-each select="@*[name() != 'wit']">
                    <xsl:sort select="."/>
                    <xsl:variable name="attName" select="name()"/>
                    <xsl:attribute name="{{$attName}}">
                      <xsl:choose>
                        <xsl:when test="name() = 'docs'">
                          <xsl:value-of select="(current-group()[not(@wit)]//@*[name() = $attName], sum(current-group()//@*[name() = $attName]))[1]"/>
                        </xsl:when>
                        <xsl:otherwise>
                          <xsl:value-of select="sum(current-group()//@*[name() = $attName])"/>
                        </xsl:otherwise>
                      </xsl:choose>
                    </xsl:attribute>
                </xsl:for-each>
                <xsl:value-of select="current-grouping-key()"/>
            </term>
        </xsl:for-each-group>
    </xsl:template>
</xsl:stylesheet>

let $conflateTerms := transform:transform(<terms>{$commonTerms, $rdgTerms}</terms>, $conflateXSLT, ())

for $a in $conflateTerms
(:order by $a/@freq/number() descending:)
return $a
