xquery version "1.0";

(:
	A simple XQuery for an XQueryTrigger that
	logs all trigger events for which it is executed
	in the file /db/triggersLogs.xml
:)

declare namespace  xmldb="http://exist-db.org/xquery/xmldb";
declare namespace tei="http://www.tei-c.org/ns/1.0";

declare variable $local:triggerEvent external;
declare variable $local:eventType external;
declare variable $local:collectionName external;
declare variable $local:documentName external;
declare variable $local:document external;

declare variable $local:triggersLogFile := "triggersLog.xml";


(: create the log file if it does not exist :)
let $logfile := 
  if(not(doc-available($local:triggersLogFile)))then
	xmldb:store("/db", $local:triggersLogFile, <triggers/>)
  else()
let $doc := doc($local:documentName)
return
if ($local:eventType eq 'finish' and $doc//tei:rdg) then 
  let $xsl := doc(concat($local:collectionName, '/triggers/splitRdgs.xsl'))
  let $rdgs := $doc//tei:witness/@xml:id
  
  for $rdg in $rdgs
  let $params :=
    <parameters>
      <param name="t" value="#{$rdg}"/>
	</parameters>
  let $transformDoc := transform:transform($doc, $xsl, $params)
  let $docName := replace(tokenize($local:documentName, '/')[last()], '(.+)(\.[^.]+)$', concat('$1', '_', $rdg, '$2'))
  let $storeDoc :=  xmldb:store($local:collectionName, $docName, $transformDoc) 
  return update insert <trigger event="{$local:triggerEvent}" eventType="{$local:eventType}" collectionName="{$local:collectionName}" documentName="{$local:documentName}" timestamp="{current-dateTime()}">{$rdg}</trigger>  into doc("/db/triggersLog.xml")/triggers
else ()